@@ -25,20 +25,7 @@ DEALINGS IN THE SOFTWARE.
 
 from __future__ import annotations
 
-from typing import (
-    TYPE_CHECKING,
-    Any,
-    Callable,
-    ClassVar,
-    Dict,
-    Generator,
-    List,
-    Optional,
-    Tuple,
-    Type,
-    TypeVar,
-    Union,
-)
+from typing import TYPE_CHECKING, Any, Callable, ClassVar, Generator, TypeVar
 
 from . import enums, utils
 from .asset import Asset
@@ -88,27 +75,31 @@ def _transform_snowflake(entry: AuditLogEntry, data: Snowflake) -> int:
     return int(data)
 
 
-def _transform_channel(entry: AuditLogEntry, data: Optional[Snowflake]) -> Optional[Union[abc.GuildChannel, Object]]:
+def _transform_channel(
+    entry: AuditLogEntry, data: Snowflake | None
+) -> abc.GuildChannel | Object | None:
     if data is None:
         return None
     return entry.guild.get_channel(int(data)) or Object(id=data)
 
 
-def _transform_member_id(entry: AuditLogEntry, data: Optional[Snowflake]) -> Union[Member, User, None]:
+def _transform_member_id(
+    entry: AuditLogEntry, data: Snowflake | None
+) -> Member | User | None:
     if data is None:
         return None
     return entry._get_member(int(data))
 
 
-def _transform_guild_id(entry: AuditLogEntry, data: Optional[Snowflake]) -> Optional[Guild]:
+def _transform_guild_id(entry: AuditLogEntry, data: Snowflake | None) -> Guild | None:
     if data is None:
         return None
     return entry._state._get_guild(data)
 
 
 def _transform_overwrites(
-    entry: AuditLogEntry, data: List[PermissionOverwritePayload]
-) -> List[Tuple[Object, PermissionOverwrite]]:
+    entry: AuditLogEntry, data: list[PermissionOverwritePayload]
+) -> list[tuple[Object, PermissionOverwrite]]:
     overwrites = []
     for elem in data:
         allow = Permissions(int(elem["allow"]))
@@ -131,19 +122,21 @@ def _transform_overwrites(
     return overwrites
 
 
-def _transform_icon(entry: AuditLogEntry, data: Optional[str]) -> Optional[Asset]:
+def _transform_icon(entry: AuditLogEntry, data: str | None) -> Asset | None:
     if data is None:
         return None
     return Asset._from_guild_icon(entry._state, entry.guild.id, data)
 
 
-def _transform_avatar(entry: AuditLogEntry, data: Optional[str]) -> Optional[Asset]:
+def _transform_avatar(entry: AuditLogEntry, data: str | None) -> Asset | None:
     if data is None:
         return None
     return Asset._from_avatar(entry._state, entry._target_id, data)  # type: ignore
 
 
-def _transform_scheduled_event_cover(entry: AuditLogEntry, data: Optional[str]) -> Optional[Asset]:
+def _transform_scheduled_event_cover(
+    entry: AuditLogEntry, data: str | None
+) -> Asset | None:
     if data is None:
         return None
     return Asset._from_scheduled_event_cover(entry._state, entry._target_id, data)
@@ -151,8 +144,8 @@ def _transform_scheduled_event_cover(entry: AuditLogEntry, data: Optional[str])
 
 def _guild_hash_transformer(
     path: str,
-) -> Callable[[AuditLogEntry, Optional[str]], Optional[Asset]]:
-    def _transform(entry: AuditLogEntry, data: Optional[str]) -> Optional[Asset]:
+) -> Callable[[AuditLogEntry, str | None], Asset | None]:
+    def _transform(entry: AuditLogEntry, data: str | None) -> Asset | None:
         if data is None:
             return None
         return Asset._from_guild_image(entry._state, entry.guild.id, data, path=path)
@@ -163,14 +156,16 @@ def _guild_hash_transformer(
 T = TypeVar("T", bound=enums.Enum)
 
 
-def _enum_transformer(enum: Type[T]) -> Callable[[AuditLogEntry, int], T]:
+def _enum_transformer(enum: type[T]) -> Callable[[AuditLogEntry, int], T]:
     def _transform(entry: AuditLogEntry, data: int) -> T:
         return enums.try_enum(enum, data)
 
     return _transform
 
 
-def _transform_type(entry: AuditLogEntry, data: int) -> Union[enums.ChannelType, enums.StickerType]:
+def _transform_type(
+    entry: AuditLogEntry, data: int
+) -> enums.ChannelType | enums.StickerType:
     if entry.action.name.startswith("sticker_"):
         return enums.try_enum(enums.StickerType, data)
     else:
@@ -181,7 +176,7 @@ class AuditLogDiff:
     def __len__(self) -> int:
         return len(self.__dict__)
 
-    def __iter__(self) -> Generator[Tuple[str, Any], None, None]:
+    def __iter__(self) -> Generator[tuple[str, Any], None, None]:
         yield from self.__dict__.items()
 
     def __repr__(self) -> str:
@@ -201,7 +196,7 @@ Transformer = Callable[["AuditLogEntry", Any], Any]
 
 
 class AuditLogChanges:
-    TRANSFORMERS: ClassVar[Dict[str, Tuple[Optional[str], Optional[Transformer]]]] = {
+    TRANSFORMERS: ClassVar[dict[str, tuple[str | None, Transformer | None]]] = {
         "verification_level": (None, _enum_transformer(enums.VerificationLevel)),
         "explicit_content_filter": (None, _enum_transformer(enums.ContentFilter)),
         "allow": (None, _transform_permissions),
@@ -250,7 +245,7 @@ class AuditLogChanges:
     def __init__(
         self,
         entry: AuditLogEntry,
-        data: List[AuditLogChangePayload],
+        data: list[AuditLogChangePayload],
         *,
         state: ConnectionState,
     ):
@@ -276,7 +271,7 @@ class AuditLogChanges:
                 if key:
                     attr = key
 
-            transformer: Optional[Transformer]
+            transformer: Transformer | None
 
             try:
                 before = elem["old_value"]
@@ -289,10 +284,15 @@ class AuditLogChanges:
             if attr == "location" and hasattr(self.before, "location_type"):
                 from .scheduled_events import ScheduledEventLocation
 
-                if self.before.location_type is enums.ScheduledEventLocationType.external:
+                if (
+                    self.before.location_type
+                    is enums.ScheduledEventLocationType.external
+                ):
                     before = ScheduledEventLocation(state=state, value=before)
                 elif hasattr(self.before, "channel"):
-                    before = ScheduledEventLocation(state=state, value=self.before.channel)
+                    before = ScheduledEventLocation(
+                        state=state, value=self.before.channel
+                    )
 
             setattr(self.before, attr, before)
 
@@ -307,10 +307,15 @@ class AuditLogChanges:
             if attr == "location" and hasattr(self.after, "location_type"):
                 from .scheduled_events import ScheduledEventLocation
 
-                if self.after.location_type is enums.ScheduledEventLocationType.external:
+                if (
+                    self.after.location_type
+                    is enums.ScheduledEventLocationType.external
+                ):
                     after = ScheduledEventLocation(state=state, value=after)
                 elif hasattr(self.after, "channel"):
-                    after = ScheduledEventLocation(state=state, value=self.after.channel)
+                    after = ScheduledEventLocation(
+                        state=state, value=self.after.channel
+                    )
 
             setattr(self.after, attr, after)
 
@@ -330,7 +335,7 @@ class AuditLogChanges:
         first: AuditLogDiff,
         second: AuditLogDiff,
         entry: AuditLogEntry,
-        elem: List[RolePayload],
+        elem: list[RolePayload],
     ) -> None:
         if not hasattr(first, "roles"):
             setattr(first, "roles", [])
@@ -412,12 +417,14 @@ class AuditLogEntry(Hashable):
         The reason this action was done.
     extra: Any
         Extra information that this entry has that might be useful.
-        For most actions, this is ``None``. However in some cases it
+        For most actions, this is ``None``. However, in some cases it
         contains extra information. See :class:`AuditLogAction` for
         which actions have this field filled out.
     """
 
-    def __init__(self, *, users: Dict[int, User], data: AuditLogEntryPayload, guild: Guild):
+    def __init__(
+        self, *, users: dict[int, User], data: AuditLogEntryPayload, guild: Guild
+    ):
         self._state = guild._state
         self.guild = guild
         self._users = users
@@ -437,27 +444,38 @@ class AuditLogEntry(Hashable):
                 self.extra: _AuditLogProxyMemberPrune = type(
                     "_AuditLogProxy", (), {k: int(v) for k, v in self.extra.items()}
                 )()
-            elif self.action is enums.AuditLogAction.member_move or self.action is enums.AuditLogAction.message_delete:
+            elif (
+                self.action is enums.AuditLogAction.member_move
+                or self.action is enums.AuditLogAction.message_delete
+            ):
                 channel_id = int(self.extra["channel_id"])
                 elems = {
                     "count": int(self.extra["count"]),
-                    "channel": self.guild.get_channel(channel_id) or Object(id=channel_id),
+                    "channel": self.guild.get_channel(channel_id)
+                    or Object(id=channel_id),
                 }
-                self.extra: _AuditLogProxyMemberMoveOrMessageDelete = type("_AuditLogProxy", (), elems)()
+                self.extra: _AuditLogProxyMemberMoveOrMessageDelete = type(
+                    "_AuditLogProxy", (), elems
+                )()
             elif self.action is enums.AuditLogAction.member_disconnect:
                 # The member disconnect action has a dict with some information
                 elems = {
                     "count": int(self.extra["count"]),
                 }
-                self.extra: _AuditLogProxyMemberDisconnect = type("_AuditLogProxy", (), elems)()
+                self.extra: _AuditLogProxyMemberDisconnect = type(
+                    "_AuditLogProxy", (), elems
+                )()
             elif self.action.name.endswith("pin"):
                 # the pin actions have a dict with some information
                 channel_id = int(self.extra["channel_id"])
                 elems = {
-                    "channel": self.guild.get_channel(channel_id) or Object(id=channel_id),
+                    "channel": self.guild.get_channel(channel_id)
+                    or Object(id=channel_id),
                     "message_id": int(self.extra["message_id"]),
                 }
-                self.extra: _AuditLogProxyPinAction = type("_AuditLogProxy", (), elems)()
+                self.extra: _AuditLogProxyPinAction = type(
+                    "_AuditLogProxy", (), elems
+                )()
             elif self.action.name.startswith("overwrite_"):
                 # the overwrite_ actions have a dict with some information
                 instance_id = int(self.extra["id"])
@@ -472,20 +490,25 @@ class AuditLogEntry(Hashable):
                     self.extra: Role = role
             elif self.action.name.startswith("stage_instance"):
                 channel_id = int(self.extra["channel_id"])
-                elems = {"channel": self.guild.get_channel(channel_id) or Object(id=channel_id)}
-                self.extra: _AuditLogProxyStageInstanceAction = type("_AuditLogProxy", (), elems)()
+                elems = {
+                    "channel": self.guild.get_channel(channel_id)
+                    or Object(id=channel_id)
+                }
+                self.extra: _AuditLogProxyStageInstanceAction = type(
+                    "_AuditLogProxy", (), elems
+                )()
 
-        self.extra: Union[
-            _AuditLogProxyMemberPrune,
-            _AuditLogProxyMemberMoveOrMessageDelete,
-            _AuditLogProxyMemberDisconnect,
-            _AuditLogProxyPinAction,
-            _AuditLogProxyStageInstanceAction,
-            Member,
-            User,
-            None,
-            Role,
-        ]
+        self.extra: (
+            _AuditLogProxyMemberPrune
+            | _AuditLogProxyMemberMoveOrMessageDelete
+            | _AuditLogProxyMemberDisconnect
+            | _AuditLogProxyPinAction
+            | _AuditLogProxyStageInstanceAction
+            | Member
+            | User
+            | None
+            | Role
+        )
 
         # this key is not present when the above is present, typically.
         # It's a list of { new_value: a, old_value: b, key: c }
@@ -497,7 +520,7 @@ class AuditLogEntry(Hashable):
         self.user = self._get_member(utils._get_as_snowflake(data, "user_id"))  # type: ignore
         self._target_id = utils._get_as_snowflake(data, "target_id")
 
-    def _get_member(self, user_id: int) -> Union[Member, User, None]:
+    def _get_member(self, user_id: int) -> Member | User | None:
         return self.guild.get_member(user_id) or self._users.get(user_id)
 
     def __repr__(self) -> str:
@@ -511,20 +534,20 @@ class AuditLogEntry(Hashable):
     @utils.cached_property
     def target(
         self,
-    ) -> Union[
-        Guild,
-        abc.GuildChannel,
-        Member,
-        User,
-        Role,
-        Invite,
-        Emoji,
-        StageInstance,
-        GuildSticker,
-        Thread,
-        Object,
-        None,
-    ]:
+    ) -> (
+        Guild
+        | abc.GuildChannel
+        | Member
+        | User
+        | Role
+        | Invite
+        | Emoji
+        | StageInstance
+        | GuildSticker
+        | Thread
+        | Object
+        | None
+    ):
         try:
             converter = getattr(self, f"_convert_target_{self.action.target_type}")
         except AttributeError:
@@ -557,19 +580,23 @@ class AuditLogEntry(Hashable):
     def _convert_target_guild(self, target_id: int) -> Guild:
         return self.guild
 
-    def _convert_target_channel(self, target_id: int) -> Union[abc.GuildChannel, Object]:
+    def _convert_target_channel(self, target_id: int) -> abc.GuildChannel | Object:
         return self.guild.get_channel(target_id) or Object(id=target_id)
 
-    def _convert_target_user(self, target_id: int) -> Union[Member, User, None]:
+    def _convert_target_user(self, target_id: int) -> Member | User | None:
         return self._get_member(target_id)
 
-    def _convert_target_role(self, target_id: int) -> Union[Role, Object]:
+    def _convert_target_role(self, target_id: int) -> Role | Object:
         return self.guild.get_role(target_id) or Object(id=target_id)
 
     def _convert_target_invite(self, target_id: int) -> Invite:
         # invites have target_id set to null
         # so figure out which change has the full invite data
-        changeset = self.before if self.action is enums.AuditLogAction.invite_delete else self.after
+        changeset = (
+            self.before
+            if self.action is enums.AuditLogAction.invite_delete
+            else self.after
+        )
 
         fake_payload = {
             "max_age": changeset.max_age,
@@ -586,20 +613,22 @@ class AuditLogEntry(Hashable):
             pass
         return obj
 
-    def _convert_target_emoji(self, target_id: int) -> Union[Emoji, Object]:
+    def _convert_target_emoji(self, target_id: int) -> Emoji | Object:
         return self._state.get_emoji(target_id) or Object(id=target_id)
 
-    def _convert_target_message(self, target_id: int) -> Union[Member, User, None]:
+    def _convert_target_message(self, target_id: int) -> Member | User | None:
         return self._get_member(target_id)
 
-    def _convert_target_stage_instance(self, target_id: int) -> Union[StageInstance, Object]:
+    def _convert_target_stage_instance(self, target_id: int) -> StageInstance | Object:
         return self.guild.get_stage_instance(target_id) or Object(id=target_id)
 
-    def _convert_target_sticker(self, target_id: int) -> Union[GuildSticker, Object]:
+    def _convert_target_sticker(self, target_id: int) -> GuildSticker | Object:
         return self._state.get_sticker(target_id) or Object(id=target_id)
 
-    def _convert_target_thread(self, target_id: int) -> Union[Thread, Object]:
+    def _convert_target_thread(self, target_id: int) -> Thread | Object:
         return self.guild.get_thread(target_id) or Object(id=target_id)
 
-    def _convert_target_scheduled_event(self, target_id: int) -> Union[ScheduledEvent, Object]:
+    def _convert_target_scheduled_event(
+        self, target_id: int
+    ) -> ScheduledEvent | Object:
         return self.guild.get_scheduled_event(target_id) or Object(id=target_id)
