@@ -1,31 +1,36 @@
 """
     Contains base definitions for different message classes."""
 
-from typing import Any, Iterable, Union, TypeVar, Optional
+from contextlib import suppress
+from typing import Any, Set, List, Iterable, Union, TypeVar, Optional
 from datetime import timedelta, datetime
 from typeguard import check_type, typechecked
 
 from ..dtypes import *
 from ..logging.tracing import *
 from ..timing import *
-from ..exceptions import *
 from .. import misc
+from .. import client
 
 import random
+import re
+import copy
 import asyncio
+import _discord as discord
 
 
 __all__ = (
     "BaseMESSAGE",
+    "AutoCHANNEL",
 )
 
 T = TypeVar("T")
+ChannelType = Union[discord.TextChannel, discord.VoiceChannel]
 
 # Configuration
 # ----------------------
 C_PERIOD_MINIMUM_SEC = 1 # Minimal seconds the period can be
 
-
 class BaseMESSAGE:
     """
     This is the base class for all the different classes that
@@ -62,19 +67,19 @@ class BaseMESSAGE:
         * datetime - specific date & time
     """
     __slots__ = (
+        "_id",
         "period",
         "start_period",
         "end_period",
         "next_send_time",
-        "data",
+        "_data",
+        "_fbcdata",
         "update_semaphore",
         "parent",
         "remove_after",
         "_created_at"
     )
 
-    __logname__: str = "" # Used for registering SQL types and to get the message type for saving the log
-
     @typechecked
     def __init__(self,
                 start_period: Optional[Union[int, timedelta]],
@@ -97,11 +102,11 @@ class BaseMESSAGE:
 
         # Deprecated int since v2.1
         if isinstance(start_period, int):
-            trace("Using int on start_period is deprecated, use timedelta object instead.", TraceLEVELS.WARNING)
+            trace("Using int on start_period is deprecated, use timedelta object instead.", TraceLEVELS.DEPRECATED)
             start_period = timedelta(seconds=start_period)
 
         if isinstance(end_period, int):
-            trace("Using int on end_period is deprecated, use timedelta object instead.", TraceLEVELS.WARNING)
+            trace("Using int on end_period is deprecated, use timedelta object instead.", TraceLEVELS.DEPRECATED)
             end_period = timedelta(seconds=end_period)
                 
         # Clamp periods to minimum level (prevent infinite loops)
@@ -112,20 +117,57 @@ class BaseMESSAGE:
         # Deprecated bool since v2.1
         if isinstance(start_in, bool): 
             self.next_send_time = datetime.now() if start_in else datetime.now() + self.end_period
-            trace("Using bool value for 'start_in' ('start_now') parameter is deprecated. Use timedelta object instead.", TraceLEVELS.WARNING)
+            trace("Using bool value for 'start_in' ('start_now') parameter is deprecated. Use timedelta object instead.", TraceLEVELS.DEPRECATED)
         else:
             self.next_send_time = datetime.now() + start_in
 
         self.parent = None # The xGUILD object this message is in (needed for update method).
         self.remove_after = remove_after # Remove the message from the list after this
         self._created_at = datetime.now()
-        self.data = data
+        self._data = data
+        self._fbcdata = isinstance(data, _FunctionBaseCLASS)
         # Attributes created with this function will not be re-referenced to a different object
         # if the function is called again, ensuring safety (.update_method)
         misc._write_attr_once(self, "update_semaphore", asyncio.Semaphore(1))
+        # For comparing copies of the object (prevents .update from overwriting)
+        misc._write_attr_once(self, "_id", id(self)) 
 
     def __repr__(self) -> str:
-        return f"{type(self).__name__}(data={self.data})"
+        return f"{type(self).__name__}(data={self._data})"
+
+    def __eq__(self, o: object) -> bool:
+        """
+        Compares two message objects.
+
+        Parameters
+        ------------
+        o: BaseMESSAGE
+            The message to compare this instance with.
+
+        Raises
+        ----------
+        TypeError
+            Parameter of incorrect type.
+        """
+        if isinstance(o, BaseMESSAGE):
+            return o._id == self._id
+        
+        raise TypeError(f"Comparison of {type(self)} not allowed with {type(o)}")
+
+    def __deepcopy__(self, *args):
+        "Duplicates the object (for use in AutoGUILD)"
+        new = copy.copy(self)
+        for slot in list(self.__slots__) + list(BaseMESSAGE.__slots__):
+            self_val = getattr(self, slot)
+            if isinstance(self_val, (asyncio.Semaphore, asyncio.Lock)):
+                # Hack to copy semaphores since not all of it can be copied directly
+                copied = type(self_val)(self_val._value)
+            else:
+                copied = copy.deepcopy((self_val))
+
+            setattr(new, slot, copied)
+
+        return new
 
     @property
     def created_at(self) -> datetime:
@@ -194,13 +236,19 @@ class BaseMESSAGE:
         """
         raise NotImplementedError
 
-    def _get_data(self) -> dict:
+    async def _get_data(self) -> dict:
         """
         Returns a dictionary of keyword arguments that is then expanded
         into other functions (_send_channel, _generate_log)
-        This is to be implemented in inherited classes due to different data_types
+        This is to be additionally implemented in inherited classes due to different data_types
+
+        .. versionchanged:: v2.3
+            Turned async.
         """
-        raise NotImplementedError
+        if self._fbcdata:
+            return await self._data.retrieve()
+
+        return self._data
 
     def _handle_error(self) -> bool:
         """
@@ -278,3 +326,155 @@ class BaseMESSAGE:
         """
 
         raise NotImplementedError
+
+@misc.doc_category("Automatic generation", path="message")
+class AutoCHANNEL:
+    """
+    .. versionadded:: v2.3
+
+    Used for creating instances of automatically managed channels.
+    The objects created with this will automatically add new channels
+    at creation and dynamically while the framework is already running,
+    if they match the patterns.
+
+    .. code-block:: python
+       :caption: Usage
+       :emphasize-lines: 4
+
+       # TextMESSAGE is used here, but works for others too
+       daf.message.TextMESSAGE(
+            ..., # Other parameters
+            channels=daf.message.AutoCHANNEL(...)
+        )
+
+
+    Parameters
+    --------------
+    include_pattern: str
+        Regex pattern to match for the channel to be considered.
+    exclude_pattern: str
+        Regex pattern to match for the channel to be excluded
+        from the consideration.
+
+        .. note::
+            If both include_pattern and exclude_pattern yield a match, the guild will be
+            excluded from match.
+
+    interval: Optional[timedelta] = timedelta(minutes=5) 
+        Interval at which to scan for new channels.
+    """
+
+    __slots__ = (
+        "include_pattern",
+        "exclude_pattern",
+        "interval",
+        "parent",
+        "cache",
+        "channel_getter",
+        "last_scan"
+    )
+    def __init__(self,
+                 include_pattern: str,
+                 exclude_pattern: Optional[str] = None,
+                 interval: Optional[timedelta] = timedelta(minutes=5)) -> None:
+        self.include_pattern = include_pattern
+        self.exclude_pattern = exclude_pattern
+        self.interval = interval.total_seconds()
+        self.parent = None
+        self.channel_getter: property = None
+        self.cache: Set[ChannelType] = set()
+        self.last_scan = 0
+
+    def __iter__(self):
+        "Returns the channel iterator."
+        self._process()
+        return iter(self.channels)
+
+    def __bool__(self) -> bool:
+        "Prevents removal of xMESSAGE by always returning True"
+        return True
+
+    @property
+    def channels(self) -> List[ChannelType]:
+        """
+        Property that returns a list of :class:`discord.TextChannel` or :class:`discord.VoiceChannel`
+        (depends on the xMESSAGE type this is in) objects in cache.
+        """
+        return list(self.cache)
+
+    async def initialize(self, parent, channel_type: str):
+        """
+        Initializes async parts of the instance.
+        This method should be called by ``parent``.
+
+        Parameters
+        -----------
+        parent: message.BaseMESSAGE
+            The message object this AutoCHANNEL instance is in.
+        channel_type: str
+            The channel type to look for when searching for channels
+        """
+        self.parent = parent
+        self.channel_getter = channel_type
+
+    def _process(self):
+        """
+        Task that scans and adds new channels to cache.
+        """
+        channel: ChannelType
+        stamp = datetime.now().timestamp()
+        if stamp - self.last_scan > self.interval:
+            self.last_scan = stamp
+            guild: discord.Guild = self.parent.parent.apiobject
+            client_: discord.Client = client.get_client()
+            member = guild.get_member(client_.user.id)
+            if member is None:
+                return
+
+            for channel in getattr(guild, self.channel_getter):
+                if channel not in self.cache:
+                    perms = channel.permissions_for(member)
+                    name = channel.name
+                    if ((perms.send_messages or (perms.connect and perms.stream and perms.speak)) and
+                        re.search(self.include_pattern, name) is not None and
+                        (self.exclude_pattern is None or re.search(self.exclude_pattern, name) is None)
+                    ):
+                        self.cache.add(channel)
+
+    def remove(self, channel: ChannelType):
+        """
+        Removes channel from cache.
+
+        Parameters
+        -----------
+        channel: Union[discord.TextChannel, discord.VoiceChannel]
+            The channel to remove from cache.
+        
+        Raises
+        -----------
+        KeyError
+            The channel is not in cache.
+        """
+        self.cache.remove(channel)
+
+    async def update(self, **kwargs):
+        """
+        Updates the object with new initialization parameters.
+
+        Parameters
+        ------------
+        kwargs: Any
+            Any number of keyword arguments that appear in the
+            object initialization.
+
+        Raises
+        -----------
+        Any
+            Raised from :py:meth:`~daf.message.AutoCHANNEL.initialize` method.
+        """
+        if "interval" not in kwargs:
+            kwargs["interval"] = timedelta(seconds=self.interval)
+        
+        return await misc._update(self,
+                                  init_options={"parent": self.parent, "channel_type": self.channel_getter},
+                                  **kwargs)
